import 'dart:math';

import 'package:clean_android_design/src/router/split_router_delegate.dart';
import 'package:clean_android_design/src/settings/clean_android_design_style.dart';
import 'package:flutter/material.dart';

class DefaultDialog extends StatelessWidget {
  final Widget? title;
  final List<Widget>? buttons;
  final bool dismissible;

  const DefaultDialog(
      {Key? key, this.title, this.buttons, this.dismissible = true})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var rowChildren = <Widget>[];
    buttons!.asMap().forEach((index, button) {
      if (index != 0)
        rowChildren.add(Padding(
          padding: const EdgeInsets.symmetric(vertical: 3),
          child: VerticalDivider(
            thickness: 1,
          ),
        ));
      rowChildren.add(button);
    });

    return WillPopScope(
      onWillPop: () {
        if (dismissible) {
          SplitRouterDelegate.instance.pop();
        }
        return Future.value(false);
      },
      child: Stack(
        children: <Widget>[
          GestureDetector(
            onTap: dismissible
                ? () {
                    SplitRouterDelegate.instance.pop();
                  }
                : null,
            child: Container(
              color: DefaultColors.background.withOpacity(0),
              width: MediaQuery.of(context).size.width,
              height: MediaQuery.of(context).size.height,
            ),
          ),
          Column(
            children: <Widget>[
              Expanded(child: Container()),
              Container(
                width: min(800, MediaQuery.of(context).size.width),
                padding: EdgeInsets.symmetric(horizontal: 30, vertical: 20),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(30),
                  color: DefaultColors.backgroundItem,
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    SizedBox(
                      height: 10,
                    ),
                    DefaultTextStyle(
                        style: TextStyles.textStyleSize(16), child: title!),
                    SizedBox(
                      height: 20,
                    ),
                    IntrinsicHeight(child: Row(children: rowChildren)),
                  ],
                ),
              ),
              AnimatedContainer(
                  duration: Duration(milliseconds: 300),
                  curve: Curves.easeOut,
                  height:
                      max(MediaQuery.of(context).viewInsets.bottom + 10, 20)),
            ],
          ),
        ],
      ),
    );
  }
}
