import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class DeleteQuestionDialog extends StatefulWidget {
  final VoidCallback onSuccess;

  final String deleteWord;
  final int count;
  final List<String>? whatsDeleted;

  const DeleteQuestionDialog({
    Key? key,
    required this.onSuccess,
    this.deleteWord = "Löschen",
    this.count = 1,
    this.whatsDeleted,
  }) : super(key: key);

  @override
  State<DeleteQuestionDialog> createState() => _DeleteQuestionDialogState();
}

class _DeleteQuestionDialogState extends State<DeleteQuestionDialog> {
  @override
  Widget build(BuildContext context) {
    var controller = TextEditingController();
    var focus = FocusNode();

    return DefaultDialog(
      title: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Container(
            decoration: BoxDecoration(
              color: Color.lerp(
                  DefaultColors.error, DefaultColors.backgroundItem, 0.9),
              borderRadius: BorderRadius.circular(5),
            ),
            padding: EdgeInsets.all(8),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Icon(Icons.warning, color: DefaultColors.error),
                    Container(width: 10),
                    Text(
                      widget.count == 1
                          ? "Diesen Eintrag löschen?"
                          : "Diese ${widget.count} Einträge löschen?",
                      style: TextStyle(
                        color: DefaultColors.error,
                        fontSize: 20,
                        fontWeight: FontWeight.w500,
                      ),
                    ),
                  ],
                ),
                Container(height: 5),
                Text(
                  "Hierdurch werden alle Daten ${widget.count == 1 ? 'des Eintrags' : 'der Einträge'} endgültig gelöscht und können nicht widerhergestellt werden",
                  style: TextStyle(
                    color: DefaultColors.error,
                    fontWeight: FontWeight.w500,
                    fontSize: 14,
                  ),
                ),
              ],
            ),
          ),
          Container(height: 30),
          widget.whatsDeleted != null
              ? Padding(
                  padding: const EdgeInsets.only(bottom: 20),
                  child: Text.rich(
                      TextSpan(style: TextStyle(fontSize: 15), children: [
                    TextSpan(
                      text: "Was wird gelöscht?",
                      style: TextStyle(
                        fontWeight: FontWeight.bold,
                        color: DefaultColors.text,
                      ),
                    ),
                    TextSpan(
                      text: widget.whatsDeleted!.map((x) => "\n- $x").join(),
                      style: TextStyle(
                        color: DefaultColors.textLight,
                      ),
                    ),
                  ])),
                )
              : Container(),
          Text.rich(TextSpan(style: TextStyle(fontSize: 15), children: [
            TextSpan(
              text:
                  "Bestätige, dass du ${widget.count == 1 ? 'diesen Eintrag' : 'diese Einträge'} wirklich löschen möchtest. Gebe dazu folgendes Wort ein: ",
              style: TextStyle(
                color: DefaultColors.textLight,
              ),
            ),
            TextSpan(
              text: widget.deleteWord,
              style: TextStyle(
                fontWeight: FontWeight.bold,
                color: DefaultColors.text,
              ),
            ),
          ])),
          Container(height: 30),
          Container(
            width: 200,
            height: 35,
            padding: EdgeInsets.only(left: 10),
            decoration: BoxDecoration(
              border: Border.all(color: DefaultColors.error, width: 2),
              borderRadius: BorderRadius.circular(5),
            ),
            child: TextField(
              style: TextStyle(
                fontSize: 16,
              ),
              textCapitalization: TextCapitalization.sentences,
              focusNode: focus,
              textAlignVertical: TextAlignVertical.center,
              controller: controller,
              decoration: InputDecoration(
                hintText: widget.deleteWord,
                focusedBorder: InputBorder.none,
                border: InputBorder.none,
                enabledBorder: InputBorder.none,
                disabledBorder: InputBorder.none,
              ),
            ),
          ),
          Container(height: 30),
        ],
      ),
      buttons: [
        DefaultDialogButton(
          titel: Text("Abbrechen"),
        ),
        StatefulBuilder(builder: (context, setState) {
          controller.addListener(() => setState(() {}));
          return DefaultDialogButton(
            color: controller.text == widget.deleteWord
                ? DefaultColors.error
                : null,
            titel: Container(
              child: Text(
                "Löschen",
                style: TextStyle(
                  color: controller.text == widget.deleteWord
                      ? Colors.white
                      : DefaultColors.textLight,
                ),
              ),
            ),
            onTap: controller.text == widget.deleteWord
                ? widget.onSuccess
                : () => focus.requestFocus(),
            closeAfterTap: controller.text == widget.deleteWord,
          );
        }),
      ],
    );
  }
}
