import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:intl/date_symbol_data_local.dart';
import 'package:intl/intl.dart';

class DatetimeFieldWidget extends StatefulWidget {
  final Text? title;
  final bool? withDate;
  final bool? withTime;
  final bool? onlyMonthAndYear;
  final bool? withPicker;
  final bool? withDeletion;

  final String? emptyDateString;

  final FieldValidation? validation;
  final void Function(DateTime? value)? saveValue;
  final DateTime? Function()? getValue;

  final bool? isEditable;
  final bool? isDisabled;

  final HighlightController? highlightController;

  final FocusNode? focusNode;
  final TextInputAction? textInputAction;

  const DatetimeFieldWidget({
    Key? key,
    required Text this.title,
    this.saveValue,
    required this.getValue,
    this.withDate = true,
    this.withTime = true,
    this.validation,
    required this.isEditable,
    this.emptyDateString = "-",
    this.isDisabled = false,
    this.onlyMonthAndYear = false,
    this.withPicker = true,
    this.withDeletion = true,
    this.highlightController,
    this.focusNode,
    this.textInputAction,
  })  : assert(getValue != null),
        assert(!(isEditable ?? true) || saveValue != null),
        super(key: key);

  @override
  _DatetimeFieldWidgetState createState() => _DatetimeFieldWidgetState();
}

class _DatetimeFieldWidgetState extends State<DatetimeFieldWidget> {
  String? errorMessage;
  FocusNode? validationFocusNode;

  bool get isEditable => widget.isEditable! && !widget.isDisabled!;

  FocusNode? focusNodeD0 = FocusNode();
  var focusNodeD1 = FocusNode(debugLabel: "focusNodeD1");
  var focusNodeD2 = FocusNode(debugLabel: "focusNodeD2");
  var focusNodeD3 = FocusNode(debugLabel: "focusNodeD3", skipTraversal: true);
  var focusNodeD4 = FocusNode(debugLabel: "focusNodeD4", skipTraversal: true);
  var focusNodeD5 = FocusNode(debugLabel: "focusNodeD5", skipTraversal: true);
  var controllerD0 = TextEditingController();
  var controllerD1 = TextEditingController();
  var controllerD2 = TextEditingController();
  bool fieldD1IsEmpty = false;
  bool fieldD2IsEmpty = false;
  FocusNode? focusNodeT0 = FocusNode(debugLabel: "focusNodeT0");
  var focusNodeT1 = FocusNode(debugLabel: "focusNodeT1");
  var focusNodeT2 = FocusNode(debugLabel: "focusNodeT2", skipTraversal: true);
  var focusNodeT3 = FocusNode(debugLabel: "focusNodeT3", skipTraversal: true);
  var controllerT0 = TextEditingController();
  var controllerT1 = TextEditingController();
  bool fieldT1IsEmpty = false;

  bool validDate = true;

  DateTime? lastShowedDate;
  bool? lastIsEditable;

  @override
  void initState() {
    super.initState();

    initializeDateFormatting();

    if (widget.withDate! && widget.focusNode != null)
      focusNodeD0 = widget.focusNode;
    else if (!widget.withDate! && widget.withTime! && widget.focusNode != null)
      focusNodeT0 = widget.focusNode;

    widget.validation?.connectToField(
      onValidate: (errorMessage) {
        if (mounted) setState(() => this.errorMessage = errorMessage);
      },
      focusNode: widget.withDate! ? focusNodeD0 : focusNodeT0,
    );

    if (isEditable) setNewDateTime(widget.getValue!(), init: true);

    focusNodeD0!.addListener(() => selectAll(focusNodeD0!, controllerD0, 2));
    focusNodeD1.addListener(() => selectAll(focusNodeD1, controllerD1, 2));
    focusNodeD2.addListener(() => selectAll(focusNodeD2, controllerD2, 4));
    focusNodeT0!.addListener(() => selectAll(focusNodeT0!, controllerT0, 2));
    focusNodeT1.addListener(() => selectAll(focusNodeT1, controllerT1, 2));

    if (!kIsWeb) {
      // replaces the RawKeyboardListeners
      controllerD0.addListener(() {
        if (controllerD0.text.length == 2) {
          if (!(controllerD0.selection.baseOffset == 0 &&
              controllerD0.selection.extentOffset == 2))
            FocusScope.of(context).requestFocus(focusNodeD1);
        }
        parseDateTimeControllers();
      });

      controllerD1.addListener(() {
        if (controllerD1.text.length == 2) {
          if (!(controllerD1.selection.baseOffset == 0 &&
              controllerD1.selection.extentOffset == 2))
            FocusScope.of(context).requestFocus(focusNodeD2);
        }
        if (controllerD1.text.length == 0) {
          fieldD1IsEmpty = true;
        } else {
          fieldD1IsEmpty = false;
        }
        parseDateTimeControllers();
      });

      controllerD2.addListener(() {
        if (controllerD2.text.length == 0) {
          fieldD2IsEmpty = true;
        } else {
          fieldD2IsEmpty = false;
        }
        parseDateTimeControllers();
      });

      controllerT0.addListener(() {
        if (controllerT0.text.length == 2) {
          if (!(controllerT0.selection.baseOffset == 0 &&
              controllerT0.selection.extentOffset == 2))
            FocusScope.of(context).requestFocus(focusNodeT1);
        }
        parseDateTimeControllers();
      });

      controllerT1.addListener(() {
        if (controllerT1.text.length == 0) {
          fieldT1IsEmpty = true;
        } else {
          fieldT1IsEmpty = false;
        }
        parseDateTimeControllers();
      });
    }
  }

  void selectAll(
      FocusNode focusNode, TextEditingController controller, int count) {
    if (focusNode.hasFocus && controller.text.length == count)
      controller.value = controller.value.copyWith(
        text: controller.text,
        selection:
            TextSelection(baseOffset: 0, extentOffset: controller.text.length),
        composing: TextRange.empty,
      );
  }

  void parseDateTimeControllers() {
    var minuteString = widget.withTime!
        ? (controllerT1.text.length == 2
            ? controllerT1.text
            : ("0" + controllerT1.text))
        : "00";
    var hourString = widget.withTime!
        ? (controllerT0.text.length == 2
            ? controllerT0.text
            : ("0" + controllerT0.text))
        : "00";
    var dayString = (widget.withDate! && !widget.onlyMonthAndYear!)
        ? (controllerD0.text.length == 2
            ? controllerD0.text
            : ("0" + controllerD0.text))
        : "01";
    var monthString = widget.withDate!
        ? (controllerD1.text.length == 2
            ? controllerD1.text
            : ("0" + controllerD1.text))
        : "01";
    var yearString = widget.withDate!
        ? (controllerD2.text.length == 4
            ? controllerD2.text
            : (controllerD2.text.length == 2 ? ("20" + controllerD2.text) : ""))
        : "2000";
    var s = "$dayString.$monthString.$yearString $minuteString:$hourString";
    try {
      var date = DateFormat("dd.MM.yyyy mm:HH").parse(s);
      if (DateFormat("dd.MM.yyyy mm:HH").format(date) == s) {
        widget.saveValue?.call(date);
        lastShowedDate = date;
        setState(() {
          validDate = true;
        });
      } else {
        setState(() {
          validDate = false;
        });
      }
    } on FormatException catch (_) {
      setState(() {
        validDate = false;
      });
    }
  }

  void setNewDateTime(DateTime? dateTime, {bool init = false}) {
    widget.saveValue?.call(dateTime);
    lastShowedDate = dateTime;

    controllerD0.text =
        dateTime == null ? "" : DateFormat("dd", "de_DE").format(dateTime);
    controllerD1.text =
        dateTime == null ? "" : DateFormat("MM", "de_DE").format(dateTime);
    controllerD2.text =
        dateTime == null ? "" : DateFormat("yyyy", "de_DE").format(dateTime);
    controllerT1.text =
        dateTime == null ? "" : DateFormat("mm", "de_DE").format(dateTime);
    controllerT0.text =
        dateTime == null ? "" : DateFormat("HH", "de_DE").format(dateTime);

    if (!init) setState(() {});
  }

  bool datesAreTheSame(DateTime? d0, DateTime? d1,
      {bool? compareDate = true, bool? compareTime = true}) {
    //print("d0: ${d0 == null ? 'null' : DateFormat("dd.MM.yyyy").format(d0)}");
    //print("d1: ${d1 == null ? 'null' : DateFormat("dd.MM.yyyy").format(d1)}");

    if (d0 == null || d1 == null) return true;
    if (compareDate!) {
      if (d0.year != d1.year || d0.month != d1.month || d0.day != d1.day)
        return false;
    }
    if (compareTime!) {
      if (d0.hour != d1.hour || d0.minute != d1.minute) return false;
    }
    return true;
  }

  @override
  Widget build(BuildContext context) {
    var isLast = ItemPosition.of(context)?.isLast ?? false;

    if ((lastIsEditable != isEditable) ||
        (isEditable &&
            !datesAreTheSame(lastShowedDate, widget.getValue!(),
                compareDate: widget.withDate, compareTime: widget.withTime))) {
      setNewDateTime(widget.getValue!(), init: true);
    }
    lastIsEditable = isEditable;

    return DefaultFieldWidget(
      title: widget.title,
      highlightController: widget.highlightController,
      childPadding:
          isEditable ? EdgeInsets.only(left: 30, right: 30, bottom: 3) : null,
      child: Column(
        children: <Widget>[
          Row(
            children: <Widget>[
              _getDateWidget(context, isLast),
              SizedBox(
                width: (widget.withDate! && widget.withTime!) ? 15 : 0,
              ),
              _getTimeWidget(isLast),
              (widget.withDeletion! && isEditable)
                  ? IconButton(
                      icon: Icon(
                        Icons.highlight_off,
                        color: DefaultColors.text,
                      ),
                      tooltip:
                          "${widget.withDate! ? 'Datum' : 'Uhrzeit'} entfernen",
                      focusNode: FocusNode(
                        debugLabel: "clear_date_button",
                        skipTraversal: true,
                      ),
                      onPressed: () {
                        FocusScope.of(context).unfocus();
                        setNewDateTime(null);
                      },
                    )
                  : Container(),
            ],
          ),
          errorMessage != null
              ? Align(
                  alignment: Alignment.centerLeft,
                  child: Padding(
                    padding: const EdgeInsets.only(top: 6),
                    child: Text(
                      errorMessage!,
                      style: TextStyles.textStyleSize(12)
                          .copyWith(color: DefaultColors.error),
                    ),
                  ),
                )
              : Container(),
        ],
      ),
    );
  }

  Widget _getDateWidget(BuildContext context, bool isLast) {
    if (!widget.withDate!) return Container();

    var date = widget.getValue!();

    if (!isEditable) {
      var dateString = date == null
          ? widget.emptyDateString!
          : DateFormat(widget.onlyMonthAndYear! ? "MMM yyyy" : "EEE dd.MM.yyy",
                  "de_DE")
              .format(date);
      return AdaptiveText(
        dateString,
        style: TextStyles.textStyleSize(17).copyWith(
          color:
              widget.isDisabled! ? DefaultColors.textLight : DefaultColors.text,
        ),
      );
    }

    var inputBorder = (!isLast || !validDate)
        ? UnderlineInputBorder(
            borderSide: BorderSide(
                color: validDate
                    ? DefaultColors.textLight
                    : DefaultColors.error.withOpacity(0.85),
                width: 1))
        : InputBorder.none;
    var focusedBorder = UnderlineInputBorder(
        borderSide: BorderSide(
      color: validDate ? DefaultColors.accent : DefaultColors.error,
      width: 2,
    ));
    var decoration = InputDecoration(
      contentPadding: EdgeInsets.only(top: 2, bottom: 5),
      isDense: true,
      enabledBorder: inputBorder,
      disabledBorder: inputBorder,
      focusedBorder: focusedBorder,
    );

    return Row(
      children: [
        widget.onlyMonthAndYear!
            ? Container()
            : Container(
                width: 30,
                child: RawKeyboardListener(
                  focusNode: focusNodeD3,
                  onKey: kIsWeb
                      ? (value) {
                          if (controllerD0.text.length == 2) {
                            if (!(controllerD0.selection.baseOffset == 0 &&
                                controllerD0.selection.extentOffset == 2))
                              FocusScope.of(context).requestFocus(focusNodeD1);
                          }
                          parseDateTimeControllers();
                        }
                      : null,
                  child: TextField(
                    textAlign: TextAlign.center,
                    controller: controllerD0,
                    focusNode: focusNodeD0,
                    maxLength: 2,
                    cursorColor: DefaultColors.accentLight,
                    decoration: decoration.copyWith(hintText: "TT"),
                    buildCounter: (context,
                            {int? currentLength, bool? isFocused, maxLength}) =>
                        null,
                    keyboardType: TextInputType.datetime,
                    textInputAction: TextInputAction.next,
                    onSubmitted: (value) {
                      FocusScope.of(context).requestFocus(focusNodeD1);
                    },
                  ),
                ),
              ),
        widget.onlyMonthAndYear! ? Container() : Text("/"),
        Container(
          width: 30,
          child: RawKeyboardListener(
            focusNode: focusNodeD4,
            onKey: kIsWeb
                ? (value) {
                    if (controllerD1.text.length == 2) {
                      if (!(controllerD1.selection.baseOffset == 0 &&
                          controllerD1.selection.extentOffset == 2))
                        FocusScope.of(context).requestFocus(focusNodeD2);
                    }
                    if (controllerD1.text.length == 0) {
                      if (fieldD1IsEmpty &&
                          !widget.onlyMonthAndYear! &&
                          value.physicalKey == PhysicalKeyboardKey.backspace) {
                        if (controllerD0.text.length > 0)
                          controllerD0.text = controllerD0.text.substring(0, 1);
                        Future.delayed(
                            Duration.zero,
                            () => FocusScope.of(context)
                                .requestFocus(focusNodeD0));
                      }
                      fieldD1IsEmpty = true;
                    } else {
                      fieldD1IsEmpty = false;
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerD1,
              focusNode: focusNodeD1,
              maxLength: 2,
              cursorColor: DefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "MM"),
              buildCounter: (context,
                      {int? currentLength, bool? isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: TextInputAction.next,
              onSubmitted: (value) {
                FocusScope.of(context).requestFocus(focusNodeD2);
              },
            ),
          ),
        ),
        Text("/"),
        Container(
          width: 50,
          child: RawKeyboardListener(
            focusNode: focusNodeD5,
            onKey: kIsWeb
                ? (value) {
                    if (controllerD2.text.length == 4) {
                      if (!(controllerD2.selection.baseOffset == 0 &&
                          controllerD2.selection.extentOffset ==
                              4)) if (widget.withTime!)
                        FocusScope.of(context).requestFocus(focusNodeT0);
                    }

                    if (controllerD2.text.length == 0) {
                      if (fieldD2IsEmpty &&
                          value.physicalKey == PhysicalKeyboardKey.backspace) {
                        if (controllerD1.text.length > 0)
                          controllerD1.text = controllerD1.text.substring(0, 1);
                        Future.delayed(
                            Duration.zero,
                            () => FocusScope.of(context)
                                .requestFocus(focusNodeD1));
                      }
                      fieldD2IsEmpty = true;
                    } else {
                      fieldD2IsEmpty = false;
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerD2,
              focusNode: focusNodeD2,
              maxLength: 4,
              cursorColor: DefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "JJJJ"),
              buildCounter: (context,
                      {int? currentLength, bool? isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: widget.withTime!
                  ? TextInputAction.next
                  : (widget.textInputAction ?? TextInputAction.done),
              onSubmitted: (value) {
                if (widget.withTime!)
                  FocusScope.of(context).requestFocus(focusNodeT0);
              },
            ),
          ),
        ),
        widget.withPicker!
            ? IconButton(
                icon: Icon(
                  Icons.event,
                  color: DefaultColors.text,
                ),
                focusNode: FocusNode(
                  debugLabel: "calendar_button",
                  skipTraversal: true,
                ),
                tooltip: "Auswahl öffnen",
                onPressed: () {
                  FocusScope.of(context).unfocus();
                  SplitRouterDelegate.instance.pushDialog(DatetimePickerDialog(
                    datetime: date,
                    onlyMonthAndYear: widget.onlyMonthAndYear,
                    setDatetime: (date) => setNewDateTime(date),
                  ));
                },
              )
            : Container(),
      ],
    );
  }

  Widget _getTimeWidget(bool isLast) {
    if (!widget.withTime!) return Container();

    var date = widget.getValue!();

    if (!isEditable) {
      var dateString = date == null
          ? (isEditable ? "Uhrzeit" : (widget.withDate! ? "" : "-"))
          : DateFormat("HH:mm").format(date);
      return AdaptiveText(
        dateString,
        style: TextStyles.textStyleSize(17).copyWith(
          color:
              widget.isDisabled! ? DefaultColors.textLight : DefaultColors.text,
        ),
      );
    }

    var inputBorder = !isLast
        ? UnderlineInputBorder(
            borderSide: BorderSide(color: DefaultColors.textLight, width: 1))
        : InputBorder.none;
    var focusedBorder = UnderlineInputBorder(
        borderSide: BorderSide(
      color: validDate ? DefaultColors.accent : DefaultColors.error,
      width: 2,
    ));
    var decoration = InputDecoration(
      contentPadding: EdgeInsets.only(top: 2, bottom: 5),
      isDense: true,
      enabledBorder: inputBorder,
      disabledBorder: inputBorder,
      focusedBorder: focusedBorder,
    );

    return Row(
      children: [
        Container(
          width: 30,
          child: RawKeyboardListener(
            focusNode: focusNodeT2,
            onKey: kIsWeb
                ? (value) {
                    if (controllerT0.text.length == 2) {
                      if (!(controllerT0.selection.baseOffset == 0 &&
                          controllerT0.selection.extentOffset == 2)) {
                        FocusScope.of(context).requestFocus(focusNodeT1);
                      }
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerT0,
              focusNode: focusNodeT0,
              maxLength: 2,
              cursorColor: DefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "hh"),
              buildCounter: (context,
                      {int? currentLength, bool? isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: TextInputAction.next,
              onSubmitted: (value) {
                FocusScope.of(context).requestFocus(focusNodeT1);
              },
            ),
          ),
        ),
        Text(":"),
        Container(
          width: 30,
          child: RawKeyboardListener(
            focusNode: focusNodeT3,
            onKey: kIsWeb
                ? (value) {
                    if (controllerT1.text.length == 0) {
                      if (fieldT1IsEmpty &&
                          value.physicalKey == PhysicalKeyboardKey.backspace) {
                        if (controllerT0.text.length > 0)
                          controllerT0.text = controllerT0.text.substring(0, 1);
                        Future.delayed(
                            Duration.zero,
                            () => FocusScope.of(context)
                                .requestFocus(focusNodeT0));
                      }
                      fieldT1IsEmpty = true;
                    } else {
                      fieldT1IsEmpty = false;
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerT1,
              focusNode: focusNodeT1,
              maxLength: 2,
              cursorColor: DefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "mm"),
              buildCounter: (context,
                      {int? currentLength, bool? isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: widget.textInputAction ?? TextInputAction.done,
            ),
          ),
        ),
        widget.withPicker!
            ? IconButton(
                icon: Icon(
                  Icons.schedule,
                  color: DefaultColors.text,
                ),
                tooltip: "Auswahl öffnen",
                focusNode: FocusNode(
                  debugLabel: "calendar_2_button",
                  skipTraversal: true,
                ),
                onPressed: () {
                  FocusScope.of(context).unfocus();
                  SplitRouterDelegate.instance.pushDialog(DatetimePickerDialog(
                    datetime: date,
                    timePicker: true,
                    setDatetime: (date) => setNewDateTime(date),
                  ));
                },
              )
            : Container(),
      ],
    );
  }
}
