import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class DefaultFieldWidget extends StatelessWidget {
  final Widget? title;
  final Widget? child;
  final EdgeInsets? childPadding;
  final HighlightController? highlightController;

  const DefaultFieldWidget({
    Key? key,
    this.title,
    this.child,
    this.childPadding,
    this.highlightController,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (highlightController == null)
      return _buildMainWidget(context, DefaultColors.backgroundItem);
    else
      return HighlightBuilder(
        controller: highlightController,
        builder: _buildMainWidget,
        defaultColor: DefaultColors.backgroundItem,
      );
  }

  Widget _buildMainWidget(BuildContext context, Color color) {
    var isFirst = ItemPosition.of(context)?.isFirst ?? false;
    var isLast = ItemPosition.of(context)?.isLast ?? false;
    var hasSeparationLine =
        ItemPosition.of(context)?.hasSeparationLine ?? false;

    var borderRadius = BorderRadius.only(
        bottomLeft: isLast ? Radius.circular(30) : Radius.zero,
        bottomRight: isLast ? Radius.circular(30) : Radius.zero,
        topLeft: isFirst ? Radius.circular(30) : Radius.zero,
        topRight: isFirst ? Radius.circular(30) : Radius.zero);

    return Container(
      width: MediaQuery.of(context).size.width,
      decoration: BoxDecoration(
        borderRadius: borderRadius,
        color: color,
      ),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          title != null
              ? Padding(
                  padding: const EdgeInsets.only(
                      left: 30, right: 30, bottom: 3, top: 13),
                  child: DefaultTextStyle(
                      style: TextStyles.textStyleSize(14).copyWith(
                        color: DefaultColors.textLight,
                      ),
                      child: title!),
                )
              : Container(),
          Padding(
            padding: childPadding ??
                const EdgeInsets.only(left: 30, right: 30, bottom: 13, top: 6),
            child: child,
          ),
          hasSeparationLine
              ? Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 30),
                  child: Divider(
                    color: DefaultColors.backgroundOutline,
                    height: 1.1,
                    thickness: 1.1,
                  ),
                )
              : Container()
        ],
      ),
    );
  }
}
