import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';

class LinkTextFieldWidget extends StatefulWidget {
  final Text? title;

  final void Function(String?)? saveValue;
  final String Function() getValue;

  final FieldValidation? validation;
  final FocusNode? focusNode;
  final void Function(String)? onSubmit;
  final TextInputType? keyboardType;
  final TextInputAction? textInputAction;
  final bool multiLine;
  final int? maxLines;

  final bool? isEditable;

  const LinkTextFieldWidget({
    Key? key,
    this.saveValue,
    this.title,
    required this.getValue,
    this.validation,
    this.focusNode,
    this.onSubmit,
    this.keyboardType,
    this.multiLine = false,
    this.maxLines = 8,
    this.isEditable,
    this.textInputAction,
  }) : super(key: key);

  @override
  _LinkTextFieldWidgetState createState() => _LinkTextFieldWidgetState();
}

class _LinkTextFieldWidgetState<T> extends State<LinkTextFieldWidget> {
  @override
  Widget build(BuildContext context) {
    if (widget.isEditable ?? false) return _getEditableWidget();

    return DefaultFieldWidget(
      title: widget.title,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: widget.getValue().split("\n").map((line) {
          return line.startsWith("http") || line.startsWith("www")
              ? Material(
                  type: MaterialType.transparency,
                  child: InkWell(
                    onTap: () async {
                      try {
                        await launchUrl(
                          Uri.parse(line),
                          mode: LaunchMode.externalApplication,
                        );
                      } catch (e) {
                        SplitRouterDelegate.instance
                            .pushDialog(const DefaultDialog(
                          title: Text("Der Link ist leider ungültig."),
                          dismissible: true,
                          buttons: <Widget>[
                            DefaultDialogButton(
                              titel: Text("Okay"),
                              closeAfterTap: true,
                            )
                          ],
                        ));
                      }
                    },
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        Flexible(
                            child: Text(
                          line,
                          style: TextStyles.textStyleSize(16).copyWith(
                            color: DefaultColors.textLink,
                            fontStyle: FontStyle.italic,
                          ),
                        )),
                        Padding(
                          padding: const EdgeInsets.only(left: 5),
                          child: Icon(Icons.open_in_browser),
                        ),
                      ],
                    ),
                  ),
                )
              : Text(
                  line,
                  style: TextStyles.textStyleSize(17)
                      .copyWith(color: DefaultColors.text),
                );
        }).toList(),
      ),
    );
  }

  Widget _getEditableWidget() {
    return TextFieldWidget(
      saveValue: widget.saveValue,
      title: widget.title,
      getValue: widget.getValue,
      validation: widget.validation,
      focusNode: widget.focusNode,
      textInputAction: widget.textInputAction,
      onSubmit: widget.onSubmit,
      keyboardType: widget.keyboardType,
      multiLine: widget.multiLine,
      maxLines: widget.maxLines,
      isEditable: widget.isEditable,
    );
  }
}
