import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class LockNavigatorService {
  static LockNavigatorService instance = LockNavigatorService();

  LockNavigatorService() {
    lockPageStack = <PageInstance>[];
    updateKey = ForceUpdateKey();
    animateUpdateKey = ForceUpdateKey();
  }

  late List<PageInstance> lockPageStack;
  late ForceUpdateKey updateKey;
  late ForceUpdateKey animateUpdateKey;

  bool get isLocked {
    _listener();
    return lockPageStack.isNotEmpty;
  }

  bool get isLeftNavigatorLocked {
    _listener();
    return isLocked &&
        lockPageStack.last.position == SplitRouterPosition.Right &&
        !SplitRouterDelegate.instance.useOnlySingleNavigator;
  }

  void lockForTopPage() {
    var topPage = SplitRouterDelegate.instance.routePath.pageInstances
        .lastWhere((page) =>
            page.position == SplitRouterPosition.Right ||
            page.position == SplitRouterPosition.Left);
    lockPageStack.add(topPage);

    updateKey.update();
    _initListener();
  }

  void unlockTopStack() {
    if (lockPageStack.isNotEmpty) lockPageStack.removeLast();

    updateKey.update();
    _initListener();
  }

  void _initListener() {
    SplitRouterDelegate.instance.addListener(_listener);
  }

  void _listener() {
    lockPageStack = lockPageStack
        .where((page) => SplitRouterDelegate.instance.routePath.pageInstances
            .any((i) => i == page))
        .toList();
  }

  void animate({bool showSnackbar = false}) {
    if (isLocked) {
      if (isLeftNavigatorLocked && !showSnackbar)
        animateUpdateKey.update();
      else
        SnackBarController.showIconAndText(Icons.save,
            "Bitte den aktuellen Bearbeitungsvorgang abbrechen oder speichern!");
    }
  }
}
