import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class LockedNavigatorWidget extends StatefulWidget {
  final Widget? child;
  final bool forSideMenu;

  const LockedNavigatorWidget({
    Key? key,
    this.child,
    this.forSideMenu = false,
  }) : super(key: key);

  @override
  _LockedNavigatorWidgetState createState() => _LockedNavigatorWidgetState();
}

class _LockedNavigatorWidgetState extends State<LockedNavigatorWidget>
    with SingleTickerProviderStateMixin {
  late AnimationController _animationController;
  late Animation<double> _opacityTween;

  @override
  void initState() {
    super.initState();

    LockNavigatorService.instance.updateKey.addListener(() {
      if (mounted) setState(() {});
    });
    LockNavigatorService.instance.animateUpdateKey
        .addListener(() => _animationController.forward());

    _animationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 300));
    _opacityTween =
        Tween<double>(begin: 0, end: 1).animate(_animationController);
  }

  @override
  Widget build(BuildContext context) {
    return LockNavigatorService.instance.isLocked &&
            (widget.forSideMenu ||
                LockNavigatorService.instance.isLeftNavigatorLocked)
        ? MouseRegion(
            onEnter: (event) => _animationController.forward(),
            onExit: (event) => _animationController.reverse(),
            child: Stack(
              children: [
                Row(
                  children: [
                    Expanded(
                      child: Column(
                        children: [
                          Expanded(child: Container()),
                        ],
                      ),
                    ),
                  ],
                ),
                widget.child!,
                AnimatedBuilder(
                    animation: _opacityTween,
                    builder: (context, _) {
                      return Container(
                        width: _opacityTween.value == 0
                            ? 0
                            : MediaQuery.of(context).size.width,
                        height: _opacityTween.value == 0
                            ? 0
                            : MediaQuery.of(context).size.height,
                        color: DefaultColors.background
                            .withOpacity(0.7 * _opacityTween.value),
                        child: Center(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(Icons.save,
                                  size: 33,
                                  color: DefaultColors.textLight
                                      .withOpacity(_opacityTween.value)),
                              Container(height: 10),
                              Text(
                                  widget.forSideMenu
                                      ? ""
                                      : "Bitte den aktuellen Bearbeitungsvorgang",
                                  style: TextStyles.defaultTextStyle().copyWith(
                                      color: DefaultColors.textLight
                                          .withOpacity(_opacityTween.value))),
                              Text(
                                  widget.forSideMenu
                                      ? ""
                                      : "abbrechen oder speichern!",
                                  style: TextStyles.defaultTextStyle().copyWith(
                                      color: DefaultColors.textLight
                                          .withOpacity(_opacityTween.value))),
                              Container(height: 30),
                            ],
                          ),
                        ),
                      );
                    }),
              ],
            ),
          )
        : widget.child!;
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }
}
