import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class NoTransitionPage extends Page {
  final PageInstance pageInstance;

  NoTransitionPage(this.pageInstance) : super(key: pageInstance.key);

  Route createRoute(BuildContext context) {
    return PageRouteBuilder(
      settings: this,
      pageBuilder: (_, __, ___) => pageInstance.build(),
      transitionDuration: Duration.zero,
      reverseTransitionDuration: Duration.zero,
    );
  }
}

class RightPage extends Page {
  final PageInstance pageInstance;

  RightPage(this.pageInstance) : super(key: pageInstance.key);

  Route createRoute(BuildContext context) {
    var child = pageInstance.build();
    return PageRouteBuilder(
      settings: this,
      pageBuilder: (_, __, ___) => child,
      transitionsBuilder: (
        BuildContext context,
        Animation<double> animation,
        Animation<double> secondaryAnimation,
        Widget child,
      ) =>
          SlideTransition(
        position: Tween<Offset>(
          begin: const Offset(1, 0),
          end: Offset.zero,
        ).animate(animation),
        child: child,
      ),
    );
  }
}

class LeftPage extends Page {
  final PageInstance pageInstance;

  LeftPage(this.pageInstance) : super(key: pageInstance.key);

  Route createRoute(BuildContext context) {
    return MaterialPageRoute(
      settings: this,
      builder: (BuildContext context) => pageInstance.build(),
    );
  }
}

class DialogPage extends Page {
  final PageInstance pageInstance;

  DialogPage(this.pageInstance) : super(key: pageInstance.key);

  Route createRoute(BuildContext context) {
    return PageRouteBuilder(
      settings: this,
      opaque: false,
      pageBuilder: (_, __, ___) => pageInstance.build(),
      transitionsBuilder: (context, animation, secondaryAnimation, child) {
        var begin = Offset(0.0, 1.0);
        var end = Offset.zero;
        var tween =
            Tween(begin: begin, end: end).chain(CurveTween(curve: Curves.ease));
        var offsetAnimation = animation.drive(tween);

        return SlideTransition(
          position: offsetAnimation,
          child: FadeTransition(
            opacity: animation,
            child: child,
          ),
        );
      },
    );
  }
}

class BackgroundPage extends Page {
  final double opacity;
  BackgroundPage(ValueKey key, this.opacity) : super(key: key);

  Route createRoute(BuildContext context) {
    return PageRouteBuilder(
      settings: this,
      opaque: false,
      pageBuilder: (
        _,
        __,
        ___,
      ) =>
          Container(
        color: DefaultColors.text.withOpacity(opacity),
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
      ),
      transitionsBuilder: (context, animation, secondaryAnimation, child) =>
          FadeTransition(
        opacity: animation,
        child: child,
      ),
    );
  }
}
