import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class MaterialAppRouter extends StatefulWidget {
  final PageCollectionMap pageCollectionMap;
  final void Function(PathTransformerCollection pathTransformerCollection)?
      modifyPathTransformerCollection;

  final Widget Function(BuildContext context, SplitRoutePath path)
      sideMenuWidgetBuilder;
  final Widget Function(BuildContext context) loadingWidgetBuilder;

  const MaterialAppRouter({
    Key? key,
    required this.pageCollectionMap,
    this.modifyPathTransformerCollection,
    required this.sideMenuWidgetBuilder,
    required this.loadingWidgetBuilder,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _MaterialAppRouterState();
}

class _MaterialAppRouterState extends State<MaterialAppRouter> {
  @override
  void initState() {
    super.initState();

    SplitRouterDelegate.newInstance(
      pageCollectionMap: widget.pageCollectionMap,
      modifyPathTransformerCollection: widget.modifyPathTransformerCollection,
      sideMenuWidgetBuilder: widget.sideMenuWidgetBuilder,
      loadingWidgetBuilder: widget.loadingWidgetBuilder,
      keepRoutePath: true,
    );

    SplitRouteInformationParser.newInstance(
      pageCollectionMap: widget.pageCollectionMap,
    );
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp.router(
      title: 'WT Admin - KampfkunstLive',
      routerDelegate: SplitRouterDelegate.instance,
      routeInformationParser: SplitRouteInformationParser.instance,
      scaffoldMessengerKey: SnackBarController.rootScaffoldMessengerKey =
          GlobalKey<ScaffoldMessengerState>(),
      theme: ThemeData(
        primaryColor: DefaultColors.instanceLight.accent,
        fontFamily: "Roboto",
        colorScheme: ColorScheme.fromSwatch().copyWith(
          secondary: DefaultColors.instanceLight.accent,
          brightness: Brightness.light,
        ),
      ),
      darkTheme: ThemeData(
        primaryColor: DefaultColors.instanceDark.accent,
        fontFamily: "Roboto",
        colorScheme: ColorScheme.fromSwatch().copyWith(
          secondary: DefaultColors.instanceDark.accent,
          brightness: Brightness.dark,
        ),
        snackBarTheme: SnackBarThemeData(
          actionTextColor: DefaultColors.instanceDark.text,
          contentTextStyle: TextStyle(color: DefaultColors.instanceDark.text),
          backgroundColor: DefaultColors.instanceDark.backgroundItem,
          disabledActionTextColor: DefaultColors.instanceDark.textLight,
        ),
      ),
      themeMode: DefaultColors.isDarkModeOn ? ThemeMode.dark : ThemeMode.light,
    );
  }
}
