import 'dart:async';
import 'dart:math';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/router/custom_pages.dart';
import 'package:flutter/material.dart';

class DialogNavigatorWidget extends StatefulWidget {
  const DialogNavigatorWidget({
    Key? key,
    required this.routePath,
    required this.notifyListeners,
    this.navigatorKey,
  }) : super(key: key);

  final SplitRoutePath routePath;
  final void Function() notifyListeners;
  final GlobalKey<NavigatorState>? navigatorKey;

  @override
  State<DialogNavigatorWidget> createState() => _DialogNavigatorWidgetState();
}

class _DialogNavigatorWidgetState extends State<DialogNavigatorWidget> {
  final ValueKey backgroundKey = ValueKey("dialog-background-key");

  bool currentlyShowDialog = false;
  Timer? timer;

  @override
  Widget build(BuildContext context) {
    var showDialog = widget.routePath.pageInstances
        .any((x) => x.position == SplitRouterPosition.Dialog);

    if (!showDialog) {
      currentlyShowDialog = false;
      timer?.cancel();
      timer = null;
      return Container();
    }

    if (showDialog && !currentlyShowDialog && timer == null) {
      timer = Timer(Duration(milliseconds: 00), () {
        setState(() {
          currentlyShowDialog = true;
          timer = null;
        });
      });
    }

    return HeroControllerScope(
      controller: HeroController(),
      child: Navigator(
        key: widget.navigatorKey ?? GlobalKey<NavigatorState>(),
        pages: [
          MaterialPage(
            key: ValueKey('Dialog Bottom'),
            child: Container(),
          ),
          BackgroundPage(ValueKey(Random().nextInt(5060).toString()), 0.3),
          ...currentlyShowDialog
              ? widget.routePath.pageInstances
                  .where((i) => i.position == SplitRouterPosition.Dialog)
                  .toList()
                  .reversed
                  .toList()
                  .sublist(1)
                  .reversed
                  .map((i) => DialogPage(i))
                  .toList()
              : [],
          BackgroundPage(ValueKey(Random().nextInt(5060).toString()), 0.4),
          ...currentlyShowDialog
              ? [
                  DialogPage(widget.routePath.pageInstances
                      .where((i) => i.position == SplitRouterPosition.Dialog)
                      .last)
                ]
              : [],
        ],
        onPopPage: (route, result) {
          if (!route.didPop(result)) {
            return false;
          }

          SplitRouterDelegate.instance.pop(SplitRouterPosition.Dialog);
          widget.notifyListeners();

          return true;
        },
      ),
    );
  }
}
