import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/router/custom_pages.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class MultipleNavigatorWidget extends StatefulWidget {
  MultipleNavigatorWidget({
    Key? key,
    required this.navigatorKeyLeft,
    required this.navigatorKeyRight,
    required this.routePath,
    required this.notifyListeners,
    required this.sideMenuWidgetBuilder,
  }) : super(key: key);

  final GlobalKey<NavigatorState> navigatorKeyLeft;
  final GlobalKey<NavigatorState> navigatorKeyRight;
  final SplitRoutePath routePath;
  final void Function() notifyListeners;
  final Widget Function(BuildContext context, SplitRoutePath path)
      sideMenuWidgetBuilder;

  @override
  State<MultipleNavigatorWidget> createState() =>
      _MultipleNavigatorWidgetState();
}

class _MultipleNavigatorWidgetState extends State<MultipleNavigatorWidget> {
  @override
  Widget build(BuildContext context) {
    PageCache.instance.clean(widget.routePath.pageInstances);

    var isIOSWeb = kIsWeb && Theme.of(context).platform == TargetPlatform.iOS;

    return Flex(
      direction: Axis.horizontal,
      children: [
        widget.sideMenuWidgetBuilder.call(context, widget.routePath),
        Expanded(
          child: HeroControllerScope(
            controller: HeroController(),
            child: LockedNavigatorWidget(
              child: Navigator(
                key: widget.navigatorKeyLeft,
                pages: widget.routePath.pageInstances
                    .where((i) => i.position == SplitRouterPosition.Left)
                    .map((i) => PageCache.instance.getCachedOrCreate(
                        i, () => isIOSWeb ? NoTransitionPage(i) : LeftPage(i)))
                    .toList(),
                onPopPage: (route, result) {
                  if (!route.didPop(result)) {
                    return false;
                  }

                  SplitRouterDelegate.instance.pop(SplitRouterPosition.Left);
                  widget.notifyListeners();

                  return true;
                },
              ),
            ),
          ),
        ),
        Container(
          width: 20,
          height: MediaQuery.of(context).size.height,
          color: DefaultColors.background,
        ),
        AnimatedContainer(
          duration: Duration(milliseconds: 150),
          width: widget.routePath.pageInstances
                  .any((i) => i.position == SplitRouterPosition.Right)
              ? (MediaQuery.of(context).size.width / 2)
              : 0,
          child: Navigator(
            key: widget.navigatorKeyRight,
            pages: [
              MaterialPage(
                key: ValueKey('SecondPage Bottom'),
                child: Container(
                  color: DefaultColors.background,
                  child: FittedBox(
                    child: Container(),
                    fit: BoxFit.fill,
                  ),
                ),
              ),
              ...widget.routePath.pageInstances
                  .where((i) => i.position == SplitRouterPosition.Right)
                  .map((i) => PageCache.instance.getCachedOrCreate(
                      i, () => isIOSWeb ? NoTransitionPage(i) : RightPage(i)))
                  .toList()
            ],
            onPopPage: (route, result) {
              if (!route.didPop(result)) {
                return false;
              }

              SplitRouterDelegate.instance.pop();
              widget.notifyListeners();

              return true;
            },
          ),
        ),
      ],
    );
  }
}
