import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/router/custom_pages.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class SingleNavigatorWidget extends StatefulWidget {
  SingleNavigatorWidget({
    Key? key,
    required this.navigatorKey,
    required this.routePath,
    required this.notifyListeners,
    required this.pageCollectionMap,
  }) : super(key: key);

  final GlobalKey<NavigatorState> navigatorKey;
  final SplitRoutePath routePath;
  final void Function() notifyListeners;
  final PageCollectionMap pageCollectionMap;

  @override
  State<SingleNavigatorWidget> createState() => _SingleNavigatorWidgetState();
}

class _SingleNavigatorWidgetState extends State<SingleNavigatorWidget> {
  final GlobalKey<NavigatorState> navigator2Key = GlobalKey<NavigatorState>();

  @override
  void initState() {
    super.initState();

    SingleNavigatorMenuButtonController.instance._forceUpdateKey
        .addListener(() {
      if (mounted)
        Future.delayed(Duration.zero, (() {
          if (mounted) setState(() {});
        }));
    });
  }

  @override
  Widget build(BuildContext context) {
    PageCache.instance.clean(widget.routePath.pageInstances);

    var isIOSWeb = kIsWeb && Theme.of(context).platform == TargetPlatform.iOS;

    return Flex(
      direction: Axis.vertical,
      children: [
        Expanded(
          child: HeroControllerScope(
            controller: HeroController(),
            child: LockedNavigatorWidget(
              child: Navigator(
                key: widget.navigatorKey,
                pages: [
                  buildDummyPage(),
                  ...widget.routePath.pageInstances
                      .where((i) =>
                          i.position == SplitRouterPosition.Left ||
                          i.position == SplitRouterPosition.Right ||
                          i.position == SplitRouterPosition.Draggable)
                      .map((i) => PageCache.instance.getCachedOrCreate(i,
                          () => isIOSWeb ? NoTransitionPage(i) : LeftPage(i)))
                      .toList()
                ].whereNotNull(),
                onPopPage: (route, result) {
                  if (!route.didPop(result)) {
                    return false;
                  }
                  SplitRouterDelegate.instance.pop();
                  widget.notifyListeners();

                  return true;
                },
              ),
            ),
          ),
        ),
      ],
    );
  }

  Page? buildDummyPage() {
    if (widget.routePath.pageInstances.length > 0 &&
        widget.routePath.pageInstances.every((instance) =>
            instance is KeyPageInstance && instance.keyPage.bottomPage)) {
      return null;
    } else {
      return MaterialPage(
        key: ValueKey("dummy"),
        child: Text("Dummy Page"),
      );
    }
  }
}

class SingleNavigatorMenuButtonController {
  static final SingleNavigatorMenuButtonController instance =
      SingleNavigatorMenuButtonController();

  final ForceUpdateKey _forceUpdateKey = ForceUpdateKey();
  bool _showMenuButtons = false;

  bool get showMenuButtons => _showMenuButtons;
  set showMenuButtons(bool value) {
    if (value != _showMenuButtons) _forceUpdateKey.update();
    _showMenuButtons = value;
  }
}
