import 'dart:async';
import 'dart:math';
import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

abstract class PageInstance {
  SplitRouterPosition get position;
  Widget rebuild();
  ValueKey get key;

  Widget? _lastChild;
  Widget build() {
    if (_lastChild == null) {
      _lastChild = rebuild();
    }
    return _lastChild!;
  }

  Completer completer = Completer();
}

class KeyPageInstance extends PageInstance {
  final KeyPage keyPage;
  final List<String> param;
  final String _keyNumber;
  KeyPageInstance(this.keyPage, this.param)
      : _keyNumber = Random().nextInt(1000000).toString();

  SplitRouterPosition get position => keyPage.position;

  @override
  Widget rebuild() => keyPage.builder(param);

  ValueKey get key =>
      ValueKey("${keyPage.key}_${param.join("_")}_#$_keyNumber#");

  @override
  String toString() => keyPage.key;
}

class AnonymousPage extends PageInstance {
  Widget Function() _builder;
  SplitRouterPosition position;
  final String keyNumber;

  AnonymousPage(this._builder, this.position)
      : keyNumber = Random().nextInt(99999).toString().padLeft(5, "0");

  @override
  Widget rebuild() => _builder();

  ValueKey get key => ValueKey("anonymous_page_#$keyNumber#");

  @override
  String toString() => "Anonymous Page ($keyNumber)";
}
