import 'package:flutter/material.dart';
import 'package:clean_android_design/clean_android_design.dart';

abstract class PageRequirement {
  RequirementsResult fulfillRequirements();
  Future<RequirementsResult> fulfillRequirementsAsync();
  bool isEqual(PageRequirement pageRequirement);
}

class RequirementsResult {
  RequirementsResultEnum result;
  PageInstance? failPage;
  bool? useAsOverlay;

  RequirementsResult.success() : result = RequirementsResultEnum.Success;

  RequirementsResult.needAsync() : result = RequirementsResultEnum.NeedAsync;

  RequirementsResult.fail(
    PageInstance failPage, {
    bool useAsOverlay = false,
  })  : result = RequirementsResultEnum.Fail,
        this.failPage = failPage,
        this.useAsOverlay = useAsOverlay;
}

enum RequirementsResultEnum {
  Success,
  Fail,
  NeedAsync,
}

class KeyPage {
  final String key;
  final int parameterCount;
  final List<PageRequirement>? requirements;
  final bool bottomPage;
  final SplitRouterPosition position;
  final Widget Function(List<String> param) builder;

  KeyPage(
    this.key,
    this.parameterCount,
    this.builder,
    this.position, {
    this.requirements,
    this.bottomPage = false,
  });

  KeyPageInstance withParam(List<String> param) {
    assert(parameterCount == param.length,
        "The list of given parameters for page $key has the wrong length. (given: ${param.length}, needed: $parameterCount)");
    return KeyPageInstance(this, param);
  }

  KeyPageInstance withoutParam() {
    assert(parameterCount == 0,
        "The page $key was instanced without any parameters. (needed: $parameterCount)");
    return KeyPageInstance(this, []);
  }
}
