import 'package:clean_android_design/src/utils/force_update_key.dart';
import 'package:flutter/material.dart';

class TextStyles {
  static TextStyle textStyleSize(double size) {
    return defaultTextStyle().copyWith(fontSize: size);
  }

  static TextStyle defaultTextStyle() {
    return TextStyle(
      color: DefaultColors.text,
      fontWeight: FontWeight.w400,
      fontSize: 15,
      fontFamily: 'Roboto',
      decoration: TextDecoration.none,
    );
  }
}

class DefaultColors {
  static Color get background => instanceCurrent.background;
  static Color get backgroundItem => instanceCurrent.backgroundItem;
  static Color get backgroundOutline => instanceCurrent.backgroundOutline;
  static Color get backgroundElevated => instanceCurrent.backgroundElevated;
  static Color get text => instanceCurrent.text;
  static Color get textLight => instanceCurrent.textLight;
  static Color get textLink => instanceCurrent.textLink;
  static Color get accent => instanceCurrent.accent;
  static Color get accentLight => instanceCurrent.accentLight;
  static Color get error => instanceCurrent.error;
  static Color get highlight => instanceCurrent.highlight;
  static Color get selected => instanceCurrent.selected;
  static Color get shimmerBackground => instanceCurrent.shimmerBackground;
  static Color get shimmerForeground => instanceCurrent.shimmerForeground;

  static ColorCollection instanceLight = ColorCollection();
  static ColorCollection instanceDark = ColorCollection();
  static ColorCollection get instanceCurrent =>
      isDarkModeOn ? instanceDark : instanceLight;

  static bool _isDarkModeOn = false;
  static bool get isDarkModeOn => _isDarkModeOn;
  static set isDarkModeOn(bool value) {
    _isDarkModeOn = value;
    updateKey.update();
  }

  static ForceUpdateKey updateKey = ForceUpdateKey();
}

class ColorCollection {
  late Color background;
  late Color backgroundItem;
  late Color backgroundOutline;
  late Color backgroundElevated;
  late Color text;
  late Color textLight;
  late Color textLink;
  late Color accent;
  late Color accentLight;
  late Color error;
  late Color highlight;
  late Color selected;
  late Color shimmerBackground;
  late Color shimmerForeground;

  ColorCollection() {
    setColors(
      background: const Color.fromRGBO(243, 243, 243, 1),
      backgroundItem: const Color.fromRGBO(255, 255, 255, 1),
      backgroundOutline: Colors.grey[350],
      backgroundElevated: Colors.grey[600],
      text: const Color.fromRGBO(70, 70, 70, 1),
      textLight: const Color.fromRGBO(230, 230, 230, 1),
      textLink: Colors.blue[900],
      accent: const Color.fromRGBO(196, 90, 16, 1),
      accentLight: const Color.fromRGBO(214, 102, 24, 1),
      error: const Color.fromRGBO(211, 47, 47, 1),
      highlight: Colors.green,
      selected: const Color.fromRGBO(196, 90, 16, 1),
      shimmerBackground: const Color(0xFFEBEBF4),
      shimmerForeground: const Color(0xFFF4F4F4),
    );
  }

  void setColors({
    Color? background,
    Color? backgroundItem,
    Color? backgroundOutline,
    Color? backgroundElevated,
    Color? text,
    Color? textLight,
    Color? textLink,
    Color? accent,
    Color? accentLight,
    Color? error,
    Color? highlight,
    Color? selected,
    Color? shimmerBackground,
    Color? shimmerForeground,
  }) {
    this.background = background ?? this.background;
    this.backgroundItem = backgroundItem ?? this.backgroundItem;
    this.backgroundOutline = backgroundOutline ?? this.backgroundOutline;
    this.backgroundElevated = backgroundElevated ?? this.backgroundElevated;
    this.text = text ?? this.text;
    this.textLight = textLight ?? this.textLight;
    this.textLink = textLink ?? this.textLink;
    this.accent = accent ?? this.accent;
    this.accentLight = accentLight ?? this.accentLight;
    this.error = error ?? this.error;
    this.highlight = highlight ?? this.highlight;
    this.selected = selected ?? this.selected;
    this.shimmerBackground = shimmerBackground ?? this.shimmerBackground;
    this.shimmerForeground = shimmerForeground ?? this.shimmerForeground;
  }
}

class DarkModeListenerWidget extends StatefulWidget {
  final Widget Function(BuildContext context, bool isDark) builder;

  const DarkModeListenerWidget({Key? key, required this.builder})
      : super(key: key);

  @override
  State<DarkModeListenerWidget> createState() => _DarkModeListenerWidgetState();
}

class _DarkModeListenerWidgetState extends State<DarkModeListenerWidget> {
  @override
  void initState() {
    super.initState();

    DefaultColors.updateKey.addListener(() {
      if (mounted) setState(() {});
    });
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder(context, DefaultColors.isDarkModeOn);
  }
}
