import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class ItemSectionWidget extends StatefulWidget {
  final Widget? title;
  final List<Widget?>? children;

  final IndexedWidgetBuilder? itemBuilder;
  final int? itemCount;

  final ItemSectionExtendedController? controller;

  final bool useSliver;

  const ItemSectionWidget({
    Key? key,
    this.title,
    this.children,
    this.controller,
  })  : itemBuilder = null,
        itemCount = null,
        useSliver = false,
        super(key: key);

  const ItemSectionWidget.builder({
    Key? key,
    this.title,
    required this.itemBuilder,
    required this.itemCount,
    this.controller,
    this.useSliver = false,
  })  : children = null,
        super(key: key);

  @override
  _ItemSectionWidgetState createState() => _ItemSectionWidgetState();
}

class _ItemSectionWidgetState extends State<ItemSectionWidget> {
  late bool extendable;

  @override
  void initState() {
    super.initState();
    extendable = widget.controller != null;

    widget.controller?.addListener((extended) {
      if (mounted) setState(() {});
    });
  }

  @override
  Widget build(BuildContext context) {
    var isExtended = widget.controller?.extended ?? true;
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        ItemSectionTitle(
          controller: widget.controller,
          title: widget.title,
        ),
        isExtended
            ? Container(
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(30),
                  color: DefaultColors.backgroundItem,
                ),
                child: widget.itemBuilder == null
                    ? ItemColumn(
                        children: widget.children!
                            .where((item) =>
                                item != null &&
                                !(item is Container && item.child == null))
                            .toList()
                            .cast<Widget>(),
                      )
                    : ListView.builder(
                        shrinkWrap: true,
                        physics: ScrollPhysics(),
                        itemCount: widget.itemCount,
                        itemBuilder: (context, index) => ItemPosition(
                          child: widget.itemBuilder!(context, index),
                          isFirst: index == 0,
                          isLast: index == widget.itemCount! - 1,
                          hasSeparationLine: index != widget.itemCount! - 1,
                        ),
                      ),
              )
            : Container(),
      ],
    );
  }
}
