import 'package:clean_android_design/clean_android_design.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';

class ListScaffoldWidget<T extends IdItemInterface> extends StatefulWidget {
  final List<T> Function(BuildContext context) listBuilder;
  final PreferredSizeWidget? Function(BuildContext context)? appBarBuilder;
  final Widget? Function(BuildContext context)? menuBarBuilder;
  final Widget? Function(BuildContext context)? drawerBuilder;

  final Future<int>? Function()? preparationFuture;
  final List<int> Function()? dummyListPreparation;
  final PreferredSizeWidget? Function(BuildContext context)?
      appBarBuilderPreparation;
  final Widget? Function(BuildContext context)? menuBarBuilderPreparation;
  final Widget? Function(BuildContext context)? drawerBuilderPreparation;

  final WillPopCallback? onWillPop;
  final List<Widget?>? Function(BuildContext context)? hintWidgetsBuilder;

  final Widget Function(BuildContext context, T item) itemBuilder;
  final Widget Function(T)? groupTitle;

  final Widget? Function(BuildContext context)? topWidgetBuilder;
  final Widget? Function(BuildContext context)? topWidgetBuilderPreparation;
  final Widget? Function(BuildContext context)? bottomWidgetBuilder;
  final Widget? Function(BuildContext context)? bottomWidgetBuilderPreparation;
  final Widget? Function(BuildContext context)? noItemWidgetBuilder;

  final List<Comparable> Function(T)? sortProperties;
  final bool reverseSort;
  final Comparable Function(T)? groupProperty;
  final List<String> Function(T item)? searchArguments;

  final bool itemsAreSelectable;
  final IListableCollection collection;

  final bool isItemPicker;
  final bool multiSelectPicker;
  final List<DocumentReference>? selectedItems;
  final void Function(List<DocumentReference> refs)? saveRefsToItem;

  final bool withCustomScrollbar;
  final Widget Function(BuildContext context, T item)?
      customScrollbarSmallIndicatorBuilder;
  final Widget Function(BuildContext context, T item)?
      customScrollbarBigIndicatorBuilder;

  final void Function(ScrollEdge edge)? scrollEdgeListener;

  const ListScaffoldWidget.list({
    Key? key,
    this.appBarBuilder,
    this.menuBarBuilder,
    this.drawerBuilder,
    this.preparationFuture,
    this.appBarBuilderPreparation,
    this.menuBarBuilderPreparation,
    this.drawerBuilderPreparation,
    this.onWillPop,
    this.hintWidgetsBuilder,
    required this.itemBuilder,
    this.dummyListPreparation,
    this.groupTitle,
    this.topWidgetBuilder,
    this.topWidgetBuilderPreparation,
    this.bottomWidgetBuilder,
    this.bottomWidgetBuilderPreparation,
    this.noItemWidgetBuilder,
    this.sortProperties,
    this.reverseSort = false,
    this.groupProperty,
    this.itemsAreSelectable = true,
    required this.listBuilder,
    this.searchArguments,
    required this.collection,
    this.withCustomScrollbar = false,
    this.customScrollbarSmallIndicatorBuilder,
    this.customScrollbarBigIndicatorBuilder,
    this.scrollEdgeListener,
  })  : this.isItemPicker = false,
        this.multiSelectPicker = true,
        this.selectedItems = null,
        this.saveRefsToItem = null,
        super(key: key);

  const ListScaffoldWidget.picker({
    Key? key,
    required this.listBuilder,
    this.appBarBuilder,
    this.menuBarBuilder,
    this.drawerBuilder,
    this.preparationFuture,
    this.appBarBuilderPreparation,
    this.menuBarBuilderPreparation,
    this.drawerBuilderPreparation,
    this.onWillPop,
    this.hintWidgetsBuilder,
    required this.itemBuilder,
    this.dummyListPreparation,
    this.groupTitle,
    this.topWidgetBuilder,
    this.topWidgetBuilderPreparation,
    this.bottomWidgetBuilder,
    this.bottomWidgetBuilderPreparation,
    this.noItemWidgetBuilder,
    this.sortProperties,
    this.reverseSort = false,
    this.groupProperty,
    required this.multiSelectPicker,
    this.selectedItems,
    this.saveRefsToItem,
    this.searchArguments,
    required this.collection,
    this.withCustomScrollbar = false,
    this.customScrollbarSmallIndicatorBuilder,
    this.customScrollbarBigIndicatorBuilder,
    this.scrollEdgeListener,
  })  : this.isItemPicker = true,
        this.itemsAreSelectable = true,
        super(key: key);

  const ListScaffoldWidget.listOrPicker({
    Key? key,
    this.appBarBuilder,
    this.menuBarBuilder,
    this.drawerBuilder,
    this.preparationFuture,
    this.appBarBuilderPreparation,
    this.menuBarBuilderPreparation,
    this.drawerBuilderPreparation,
    this.onWillPop,
    this.hintWidgetsBuilder,
    required this.itemBuilder,
    this.dummyListPreparation,
    this.groupTitle,
    this.topWidgetBuilder,
    this.topWidgetBuilderPreparation,
    this.bottomWidgetBuilder,
    this.bottomWidgetBuilderPreparation,
    this.noItemWidgetBuilder,
    this.sortProperties,
    this.reverseSort = false,
    this.groupProperty,
    this.itemsAreSelectable = true,
    required this.listBuilder,
    this.searchArguments,
    required this.isItemPicker,
    this.multiSelectPicker = true,
    this.selectedItems,
    this.saveRefsToItem,
    required this.collection,
    this.withCustomScrollbar = false,
    this.customScrollbarSmallIndicatorBuilder,
    this.customScrollbarBigIndicatorBuilder,
    this.scrollEdgeListener,
  }) : super(key: key);

  @override
  State<ListScaffoldWidget<T>> createState() => _ListScaffoldWidgetState<T>();
}

class _ListScaffoldWidgetState<T extends IdItemInterface>
    extends State<ListScaffoldWidget<T>> {
  late ListState state;

  @override
  void initState() {
    super.initState();

    state = ListState(
      selectionIsActive: widget.isItemPicker,
      multiSelect: !widget.isItemPicker || widget.multiSelectPicker,
      selectedRefs: []
        ..addAll((widget.isItemPicker ? widget.selectedItems : null) ?? []),
    );
  }

  @override
  Widget build(BuildContext context) {
    return ListController(
      state: state,
      child: BasicScaffold(
        preparationFuture: widget.preparationFuture,
        appBarBuilder: widget.appBarBuilder,
        appBarBuilderPreparation: widget.appBarBuilderPreparation,
        drawerBuilder: widget.drawerBuilder,
        drawerBuilderPreparation: widget.drawerBuilderPreparation,
        menuBarBuilder:
            widget.isItemPicker ? buildPickerMenuBar : widget.menuBarBuilder,
        menuBarBuilderPreparation: widget.isItemPicker
            ? buildPickerMenuBar
            : widget.menuBarBuilderPreparation,
        hintWidgetsBuilder: widget.hintWidgetsBuilder,
        onWillPop: onWillPopList,
        scrollable: false,
        builder: (context) => ListContentWidget(
          listBuilder: widget.listBuilder,
          isItemPicker: widget.isItemPicker,
          itemBuilder: widget.itemBuilder,
          itemsAreSelectable: widget.itemsAreSelectable,
          multiSelectPicker: widget.multiSelectPicker,
          reverseSort: widget.reverseSort,
          groupProperty: widget.groupProperty,
          groupTitle: widget.groupTitle,
          collection: widget.collection,
          noItemWidgetBuilder: widget.noItemWidgetBuilder,
          saveRefsToItem: widget.saveRefsToItem,
          sortProperties: widget.sortProperties,
          topWidgetBuilder: widget.topWidgetBuilder,
          bottomWidgetBuilder: widget.bottomWidgetBuilder,
          searchArguments: widget.searchArguments,
          withCustomScrollbar: widget.withCustomScrollbar,
          customScrollbarBigIndicatorBuilder:
              widget.customScrollbarBigIndicatorBuilder,
          customScrollbarSmallIndicatorBuilder:
              widget.customScrollbarSmallIndicatorBuilder,
          scrollEdgeListener: widget.scrollEdgeListener,
        ),
        builderPreparation: builderPreparation,
      ),
    );
  }

  Widget? buildPickerMenuBar(BuildContext context) {
    return DefaultMenuBar(
      menuButtons: [
        SimpleScaffoldMenuButtonWidget(
          text: Text("Abbrechen"),
          onTap: () {
            Navigator.of(context).pop();
          },
          tooltip: "Auswahl abbrechen",
        ),
        widget.multiSelectPicker
            ? SimpleScaffoldMenuButtonWidget(
                text: Text("Auswählen"),
                onTap: () {
                  widget.saveRefsToItem
                      ?.call(ListController.of(context).state.selectedRefs);
                  Navigator.of(context).pop();
                },
                tooltip: "Auswahl speichern",
              )
            : null,
      ],
    );
  }

  Widget builderPreparation(BuildContext context) {
    return SingleChildScrollView(
      child: Column(
        children: [
          (widget.topWidgetBuilderPreparation != null
                  ? widget.topWidgetBuilderPreparation?.call(context)
                  : widget.topWidgetBuilder?.call(context)) ??
              Container(),
          ...(widget.dummyListPreparation?.call() ?? [5, 3])
              .map((sectionLength) => ItemSectionWidget(
                    title: Container(
                      width: 80,
                      height: 20,
                      decoration: BoxDecoration(
                        color: DefaultColors.shimmerBackground,
                        borderRadius: BorderRadius.circular(50),
                      ),
                    ),
                    children: List.generate(
                        sectionLength, (index) => ShimmerListItem()),
                  ))
              .toList(),
          (widget.bottomWidgetBuilderPreparation != null
                  ? widget.bottomWidgetBuilderPreparation?.call(context)
                  : widget.bottomWidgetBuilder?.call(context)) ??
              Container(),
        ],
      ),
    );
  }

  Future<bool> onWillPopList(BuildContext context) async {
    var state = ListController.of(context).state;
    if (state.searchIsActive) {
      state.searchIsActive = false;
      state.notifyListeners();
      return false;
    }
    if (state.selectionIsActive && !widget.isItemPicker) {
      state.selectionIsActive = false;
      state.notifyListeners();
      return false;
    }
    return true;
  }
}

class ListState with ChangeNotifier {
  final List<DocumentReference> selectedRefs;
  final bool multiSelect;
  final TextEditingController searchController = TextEditingController();
  final bool clearSearchControllerOnChange;

  ListState({
    required bool selectionIsActive,
    this.selectedRefs = const [],
    this.multiSelect = true,
    this.clearSearchControllerOnChange = true,
  }) : this._selectionIsActive = selectionIsActive;

  bool _selectionIsActive;
  bool _searchIsActive = false;

  bool get selectionIsActive => _selectionIsActive;
  bool get searchIsActive => _searchIsActive;

  List<DocumentReference>? allReferences;

  set selectionIsActive(bool value) {
    if (_selectionIsActive == value) return;
    selectedRefs.clear();
    _selectionIsActive = value;
    notifyListeners();
  }

  set searchIsActive(bool value) {
    if (_searchIsActive == value) return;
    if (clearSearchControllerOnChange) searchController.clear();
    _searchIsActive = value;
    notifyListeners();
  }

  bool toogleSelectionFor(DocumentReference ref) {
    if (!_selectionIsActive) {
      _selectionIsActive = true;
      selectedRefs.clear();
    }
    if (selectedRefs.any((x) => x.id == ref.id)) {
      selectedRefs.removeWhere((x) => x.id == ref.id);
      notifyListeners();
      return false;
    } else {
      selectedRefs.add(ref);
      notifyListeners();
      return true;
    }
  }

  void selectAll() {
    assert(allReferences != null,
        "[ListState] selectAll() was executed befor allReferences where set");
    _selectionIsActive = true;
    selectedRefs.clear();
    selectedRefs.addAll(allReferences!);
    notifyListeners();
  }

  void notifyListeners() {
    super.notifyListeners();
  }

  bool isSelected(IdItemInterface item) =>
      _selectionIsActive && selectedRefs.any((ref) => ref.id == item.id);

  List<String> get searchTexts => searchController.text.split(" ");
}

class ListController extends InheritedWidget {
  final ListState state;

  ListController({required Widget child, required this.state})
      : super(child: child);

  @override
  bool updateShouldNotify(InheritedWidget oldWidget) => true;

  static ListController of(BuildContext context) {
    final ListController? result =
        context.dependOnInheritedWidgetOfExactType<ListController>();
    assert(result != null, 'No ListController found in context');
    return result!;
  }

  static ListController? maybeOf(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<ListController>();
  }
}
