import 'package:clean_android_design/clean_android_design.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';

class SelectionMenuBar extends StatefulWidget {
  final List<Widget?>? Function(
          BuildContext context, List<DocumentReference> selectedRefs)
      menuButtonsBuilder;
  final Widget? alternativeWidget;

  const SelectionMenuBar({
    Key? key,
    required this.menuButtonsBuilder,
    this.alternativeWidget,
  }) : super(key: key);

  @override
  State<SelectionMenuBar> createState() => _SelectionMenuBarState();
}

class _SelectionMenuBarState extends State<SelectionMenuBar> {
  @override
  Widget build(BuildContext context) {
    ListController.of(context).state.removeListener(listener);
    ListController.of(context).state.addListener(listener);

    var menuButtons = widget.menuButtonsBuilder
            .call(context, ListController.of(context).state.selectedRefs)
            ?.whereNotNull() ??
        [];

    if (widget.alternativeWidget == null) {
      return DefaultMenuBar(
        menuButtons: menuButtons,
      );
    } else {
      return Container(
        height: 60,
        color: DefaultColors.background,
        child: AnimatedSwitcher(
          duration: Duration(milliseconds: 300),
          transitionBuilder: (child, animation) {
            final offsetAnimation =
                Tween<Offset>(begin: Offset(0.0, 1.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            return ClipRect(
              child: SlideTransition(
                position: offsetAnimation,
                child: child,
              ),
            );
          },
          child: menuButtons.length > 0
              ? Row(
                  key: Key("row"),
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: menuButtons,
                )
              : widget.alternativeWidget!,
        ),
      );
    }
  }

  void listener() {
    if (mounted) {
      setState(() {});
    }
  }
}
