import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class UpdateAppBar<T extends UpdateableDatabaseItem<T>> extends StatefulWidget
    implements PreferredSizeWidget {
  final Widget? Function(BuildContext context) titleBuilder;
  final Widget? Function(BuildContext context) titleLoadingItemBuilder;
  final Widget? Function(BuildContext context)? titleNewItemBuilder;

  final Widget? Function(BuildContext context)? descriptionBuilder;
  final Widget? Function(BuildContext context)? descriptionLoadingItemBuilder;
  final Widget? Function(BuildContext context)? descriptionNewItemBuilder;

  final List<Widget?>? Function(BuildContext context)? appBarButtonsBuilder;
  final List<Widget?>? Function(BuildContext context)? editAppBarButtonsBuilder;

  final List<Widget?>? Function(BuildContext context)?
      appBarPopupButtonsBuilder;
  final List<Widget?>? Function(BuildContext context)?
      editAppBarPopupButtonsBuilder;

  final bool? showBackArrow;

  const UpdateAppBar({
    Key? key,
    this.showBackArrow,
    required this.titleBuilder,
    required this.titleLoadingItemBuilder,
    this.titleNewItemBuilder,
    this.descriptionBuilder,
    this.appBarButtonsBuilder,
    this.editAppBarButtonsBuilder,
    this.descriptionLoadingItemBuilder,
    this.descriptionNewItemBuilder,
    this.appBarPopupButtonsBuilder,
    this.editAppBarPopupButtonsBuilder,
  }) : super(key: key);

  @override
  State<UpdateAppBar<T>> createState() => _UpdateAppBarState<T>();

  @override
  Size get preferredSize => AppBar(
        title: Text("Test"),
      ).preferredSize;
}

class _UpdateAppBarState<T extends UpdateableDatabaseItem<T>>
    extends State<UpdateAppBar<T>> {
  FocusNode searchFocusNode = FocusNode();

  @override
  Widget build(BuildContext context) {
    UpdateController.of<T>(context).updateNotifier.removeListener(listener);
    UpdateController.of<T>(context).updateNotifier.addListener(listener);

    var frame = UpdateController.of<T>(context).updateItemFrame;

    return DefaultAppBar(
      title: ((frame.isInitiated
                  ? (frame.editableItem.id == null
                      ? widget.titleNewItemBuilder
                      : widget.titleBuilder)
                  : widget.titleLoadingItemBuilder) ??
              widget.titleBuilder)
          .call(context),
      description: ((frame.isInitiated
                  ? (frame.editableItem.id == null
                      ? widget.descriptionNewItemBuilder
                      : widget.descriptionBuilder)
                  : widget.descriptionLoadingItemBuilder) ??
              widget.descriptionBuilder)
          ?.call(context),
      showBackArrow: widget.showBackArrow,
      appBarButtons: frame.isInitiated
          ? (frame.isEditable
              ? widget.editAppBarButtonsBuilder?.call(context)
              : widget.appBarButtonsBuilder?.call(context))
          : null,
      appBarPopupButtons: frame.isInitiated
          ? (frame.isEditable
              ? widget.editAppBarPopupButtonsBuilder?.call(context)
              : widget.appBarPopupButtonsBuilder?.call(context))
          : null,
    );
  }

  void listener() {
    if (mounted) setState(() {});
  }
}
