import 'package:clean_android_design/clean_android_design.dart';

class UpdateItemFrame<T extends UpdateableDatabaseItem<T>> {
  bool _isEditable = false;

  // if isEditable = false, all three are the same
  T? _currentDBItem;
  T? _originalItem;
  T? _editableItem;

  bool setNewItem(T item) {
    if (_currentDBItem != null && _currentDBItem!.equals(item)) return false;

    _currentDBItem = item;
    if (!isEditable) {
      _originalItem = item;
      _editableItem = item;
    }

    return true;
  }

  void setEditable(bool isEditable) {
    _isEditable = isEditable;
    if (isEditable) {
      _editableItem = _currentDBItem?.copy();
      _originalItem = _currentDBItem?.copy();
    } else {
      _editableItem = _currentDBItem;
      _originalItem = _currentDBItem;
    }
  }

  bool get isEditable => _isEditable;
  T get currentDBItem {
    assert(_currentDBItem != null,
        "[UpdateItemFrame] 'currentDBItem' was requested, even though it was not initiated");
    return _currentDBItem!;
  }

  T get originalItem {
    assert(_originalItem != null,
        "[UpdateItemFrame] 'originalItem' was requested, even though it was not initiated");
    return _originalItem!;
  }

  T get editableItem {
    assert(_editableItem != null,
        "[UpdateItemFrame] 'editableItem' was requested, even though it was not initiated");
    return _editableItem!;
  }

  bool get isInitiated => _originalItem != null;
}
