import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class UpdateMenuBar<T extends UpdateableDatabaseItem<T>>
    extends StatefulWidget {
  final bool hasEditButton;
  final bool hasDeleteButton;
  final List<Widget?>? additionalMenuButtons;
  final List<Widget?>? additionalEditingMenuButtons;
  final List<String>? whatsDeleted;

  const UpdateMenuBar({
    Key? key,
    required this.hasEditButton,
    required this.hasDeleteButton,
    this.additionalMenuButtons,
    this.additionalEditingMenuButtons,
    this.whatsDeleted,
  }) : super(key: key);

  @override
  State<UpdateMenuBar<T>> createState() => _UpdateMenuBarState<T>();
}

class _UpdateMenuBarState<T extends UpdateableDatabaseItem<T>>
    extends State<UpdateMenuBar<T>> {
  @override
  Widget build(BuildContext context) {
    UpdateController.of<T>(context).updateNotifier.removeListener(listener);
    UpdateController.of<T>(context).updateNotifier.addListener(listener);

    return DefaultMenuBar(
      menuButtons: UpdateController.of<T>(context).updateItemFrame.isEditable
          ? buildEditButtons(context)
          : buildDefaultButtons(context),
    );
  }

  List<Widget?> buildEditButtons(BuildContext context) {
    return [
      SimpleScaffoldMenuButtonWidget(
        text: Text("Abbrechen"),
        onTap: () => UpdateController.of<T>(context).abortEditing(context),
        tooltip: "Bearbeitung abbrechen",
      ),
      SimpleScaffoldMenuButtonWidget(
        text: Text("Speichern"),
        onTap: () => UpdateController.of<T>(context).trySave(context),
        tooltip: "Änderungen speichern",
      ),
      ...(widget.additionalEditingMenuButtons ?? [])
    ];
  }

  List<Widget?> buildDefaultButtons(BuildContext context) {
    return [
      widget.hasEditButton
          ? IconScaffoldMenuButtonWidget(
              icon: Icons.edit,
              text: Text("Bearbeiten"),
              onTap: () =>
                  UpdateController.of<T>(context).tryStartEditing(context),
              tooltip: "Eigenschaften bearbeiten",
            )
          : null,
      widget.hasDeleteButton
          ? IconScaffoldMenuButtonWidget(
              icon: Icons.delete,
              text: Text("Löschen"),
              tooltip: "Eintrag löschen",
              onTap: () => UpdateController.of<T>(context).tryDelete(
                context,
                whatsDeleted: widget.whatsDeleted,
              ),
            )
          : null,
      ...(widget.additionalMenuButtons ?? [])
    ];
  }

  void listener() {
    if (mounted) setState(() {});
  }
}
