import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class AdaptiveText extends StatelessWidget {
  final String? text;
  final InlineSpan? textSpan;
  final TextStyle? style;
  final bool rich;

  final alwaysSelectable;
  final alwaysPlain;

  const AdaptiveText(
    this.text, {
    Key? key,
    this.style,
    this.alwaysSelectable = false,
    this.alwaysPlain = false,
  })  : rich = false,
        textSpan = null,
        super(key: key);

  const AdaptiveText.rich(
    this.textSpan, {
    Key? key,
    this.style,
    this.alwaysSelectable = false,
    this.alwaysPlain = false,
  })  : rich = true,
        text = null,
        super(key: key);

  const AdaptiveText._copy(
    this.textSpan,
    Key? key,
    this.style,
    this.alwaysSelectable,
    this.alwaysPlain,
    this.rich,
    this.text,
  ) : super(key: key);

  AdaptiveText overrideToPlain() {
    return AdaptiveText._copy(
      textSpan,
      key,
      style,
      alwaysSelectable,
      alwaysPlain,
      rich,
      text,
    );
  }

  @override
  Widget build(BuildContext context) {
    if (rich)
      return ((kIsWeb || alwaysSelectable) && !alwaysPlain)
          ? SelectableText.rich(
              textSpan as TextSpan,
              style: style,
            )
          : Text.rich(
              textSpan!,
              style: style,
            );
    else
      return ((kIsWeb || alwaysSelectable) && !alwaysPlain)
          ? SelectableText(
              text!,
              style: style,
            )
          : Text(
              text!,
              style: style,
            );
  }
}
