import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class SimpleButtonWidget extends StatelessWidget {
  final Widget? child;
  final String? text;
  final double? width;
  final void Function()? onPressed;
  final bool enabled;
  final double? height;
  final Color? color;
  final Color? borderColor;
  final HighlightController? highlightController;
  final FocusNode? focusNode;

  const SimpleButtonWidget({
    Key? key,
    this.text,
    this.onPressed,
    this.child,
    this.enabled = true,
    this.width = 250,
    this.height = 36,
    this.color,
    this.borderColor,
    this.highlightController,
    this.focusNode,
  })  : assert(
            (child != null && text == null) || (child == null && text != null)),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return highlightController == null
        ? _buildMainWidget(context, color ?? DefaultColors.backgroundOutline)
        : HighlightBuilder(
            controller: highlightController,
            defaultColor: color ?? DefaultColors.backgroundOutline,
            builder: (context, color) => _buildMainWidget(context, color),
          );
  }

  Widget _buildMainWidget(BuildContext context, Color? color) {
    // var maxWidth = MediaQuery.of(context).size.width *
    //         (SplitRouterDelegate.instance.useOnlySingleNavigator ? 1 : 0.5) -
    //     80;
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Container(
        // width: width ?? maxWidth,
        width: width,
        height: height,
        decoration: borderColor != null
            ? BoxDecoration(
                border: Border.all(color: borderColor!.withOpacity(0.5)),
                borderRadius: BorderRadius.circular(18),
                boxShadow: [
                    BoxShadow(
                      color: borderColor!.withOpacity(0.5),
                      spreadRadius: 0.5,
                      blurRadius: 2,
                      offset: Offset(0, 1),
                    ),
                  ])
            : null,
        child: Material(
          borderRadius: BorderRadius.circular(18),
          color: color,
          child: InkWell(
            borderRadius: BorderRadius.circular(18),
            focusNode: focusNode,
            onTap: enabled && onPressed != null
                ? () {
                    FocusScope.of(context).unfocus();
                    onPressed?.call();
                  }
                : null,
            child: DefaultTextStyle(
                style: TextStyle(
                  color: enabled
                      ? DefaultColors.text
                      : Color.lerp(DefaultColors.backgroundOutline,
                          DefaultColors.text, 0.3),
                  fontSize: 15.5,
                  fontWeight: FontWeight.w500,
                ),
                child: child == null ? Center(child: Text(text!)) : child!),
          ),
        ),
      ),
    );
  }
}
