import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class HighlightPopupMenuButton<T> extends StatelessWidget {
  final HighlightController? highlightController;
  final String? tooltip;
  final void Function(T)? onSelected;
  final List<PopupMenuEntry<T>> Function(BuildContext) itemBuilder;
  final Widget? icon;

  final bool repeatUntilOpen;

  const HighlightPopupMenuButton({
    Key? key,
    this.highlightController,
    required this.itemBuilder,
    this.tooltip,
    this.onSelected,
    this.icon,
    this.repeatUntilOpen = false,
  })  : assert(!repeatUntilOpen || highlightController == null),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    var highlightController = repeatUntilOpen
        ? (HighlightController()..repeat())
        : this.highlightController;

    return HighlightBuilder(
      controller: highlightController,
      defaultColor: DefaultColors.accent.withOpacity(0),
      highlightColor: DefaultColors.accent,
      builder: (context, color) {
        return PopupMenuButton<T>(
          tooltip: tooltip,
          onSelected: onSelected,
          padding: EdgeInsets.zero,
          icon: Stack(
            children: [
              Container(
                width: 150,
                height: 150,
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(20),
                  gradient: RadialGradient(
                    colors: [color, color.withOpacity(0)],
                    center: Alignment.center,
                    radius: 0.8,
                  ),
                ),
              ),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: icon ?? Icon(Icons.more_vert),
              ),
            ],
          ),
          itemBuilder: (context) {
            if (repeatUntilOpen && highlightController!.onRepeat) {
              highlightController.highlight();
            }
            return itemBuilder(context);
          },
        );
      },
    );
  }
}
