import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class LoadingWidgetService {
  static LoadingWidgetService instance = LoadingWidgetService();

  LoadingWidgetService() {
    updateKey.addListener(() {
      _states = _states.where((x) => x._active).toList();
    });
  }

  ForceUpdateKey updateKey = ForceUpdateKey();

  LoadingWidgetServiceState show() {
    _states.add(LoadingWidgetServiceState._());
    Future.delayed(Duration.zero, () => updateKey.update());
    return _states.last;
  }

  List<LoadingWidgetServiceState> _states = <LoadingWidgetServiceState>[];
}

class LoadingWidgetServiceState {
  bool _canPop = false;
  String _message = "";
  bool _active = true;

  bool get active => _active;

  bool get canPop => _canPop;
  set canPop(bool x) {
    _canPop = x;
    LoadingWidgetService.instance.updateKey.update();
  }

  String get message => _message;
  set message(String x) {
    _message = x;
    LoadingWidgetService.instance.updateKey.update();
  }

  void deactivate() {
    _active = false;
    LoadingWidgetService.instance.updateKey.update();
  }

  LoadingWidgetServiceState._();
}

class LoadingWidget extends StatefulWidget {
  const LoadingWidget({
    Key? key,
  }) : super(key: key);

  @override
  State<LoadingWidget> createState() => _LoadingWidgetState();
}

class _LoadingWidgetState extends State<LoadingWidget> {
  @override
  void initState() {
    super.initState();

    LoadingWidgetService.instance.updateKey.addListener(() {
      if (mounted) setState(() {});
    });
  }

  @override
  Widget build(BuildContext context) {
    if (LoadingWidgetService.instance._states.isEmpty) {
      return Container();
    }
    return WillPopScope(
      onWillPop: () async => LoadingWidgetService.instance._states.last.canPop,
      child: Container(
        width: double.infinity,
        height: double.infinity,
        color: DefaultColors.background.withOpacity(0.3),
        child: Stack(
          children: [
            Align(
              alignment: Alignment.center,
              child: Container(
                width: 70,
                height: 70,
                child: CircularProgressIndicator(
                  valueColor:
                      AlwaysStoppedAnimation<Color?>(DefaultColors.textLight),
                ),
              ),
            ),
            Align(
              alignment: Alignment.center,
              child: Padding(
                padding: EdgeInsets.only(top: 150),
                child: AnimatedSwitcher(
                  duration: Duration(milliseconds: 300),
                  child: Text(
                    LoadingWidgetService.instance._states.last.message,
                    style: TextStyles.textStyleSize(16).copyWith(
                      color: DefaultColors.text,
                    ),
                    key:
                        Key(LoadingWidgetService.instance._states.last.message),
                  ),
                ),
              ),
            )
          ],
        ),
      ),
    );
  }
}

class TransparentRoute extends PageRoute<void> {
  TransparentRoute({
    required this.builder,
    RouteSettings? settings,
  }) : super(settings: settings, fullscreenDialog: false);

  final WidgetBuilder builder;

  @override
  bool get opaque => false;

  @override
  Color? get barrierColor => null;

  @override
  String? get barrierLabel => null;

  @override
  bool get maintainState => true;

  @override
  Duration get transitionDuration => Duration(milliseconds: 350);

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    final result = builder(context);
    return FadeTransition(
      opacity: Tween<double>(begin: 0, end: 1).animate(animation),
      child: Semantics(
        scopesRoute: true,
        explicitChildNodes: true,
        child: result,
      ),
    );
  }
}
