import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class SearchBar extends StatelessWidget {
  final TextEditingController controller;
  final FocusNode? focusNode;
  final bool showBackButton;

  const SearchBar({
    Key? key,
    required this.controller,
    this.focusNode,
    this.showBackButton = true,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      width: MediaQuery.of(context).size.width,
      height: 42,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(21),
        color: DefaultColors.backgroundOutline,
      ),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          showBackButton
              ? IconButton(
                  icon: Icon(
                    Icons.keyboard_arrow_left,
                    color: DefaultColors.text,
                    size: 30,
                  ),
                  onPressed: () => Navigator.of(context).maybePop(),
                )
              : Container(width: 10),
          Expanded(
            child: TextField(
              controller: controller,
              cursorColor: DefaultColors.accent,
              decoration: InputDecoration(
                hintText: "Suche",
                border: InputBorder.none,
              ),
              textCapitalization: TextCapitalization.sentences,
              textInputAction: TextInputAction.search,
              style: TextStyles.textStyleSize(18)
                  .copyWith(color: DefaultColors.text),
              focusNode: focusNode,
            ),
          ),
          ValueListenableBuilder<TextEditingValue>(
            valueListenable: controller,
            builder: (_, value, ___) => value.text != ""
                ? IconButton(
                    icon: Icon(
                      Icons.close,
                      color: DefaultColors.text,
                      size: 25,
                    ),
                    tooltip: "Suche leeren",
                    onPressed: () {
                      controller.text = "";
                      focusNode?.requestFocus();
                    },
                  )
                : Container(),
          ),
        ],
      ),
    );
  }
}
