import 'package:flutter/material.dart';

abstract class IValidation<T> {
  ValidationResult isValid(T item);
}

class Validation<T> implements IValidation<T> {
  final ValidationResult Function(T item)? _isValid;
  final bool Function(T item)? _isValidSimple;

  Validation(ValidationResult Function(T item) isValid)
      : _isValid = isValid,
        _isValidSimple = null;
  Validation.simple(bool Function(T item) isValidSimple)
      : _isValid = null,
        _isValidSimple = isValidSimple;

  @override
  ValidationResult isValid(T item) {
    if (_isValid != null)
      return _isValid!(item);
    else
      return ValidationResult(
        isValid: _isValidSimple!(item),
      );
  }
}

class ValidationResult {
  bool isValid;
  FocusNode? firstInvalidNode;
  Key? firstInvalidPage;

  ValidationResult({this.isValid = true, FocusNode? focusNode, Key? pageKey})
      : this.firstInvalidNode = focusNode,
        this.firstInvalidPage = pageKey;

  ValidationResult merge(ValidationResult result) {
    this.isValid = this.isValid && result.isValid;
    if (!result.isValid) {
      this.firstInvalidNode ??= result.firstInvalidNode;
      this.firstInvalidPage ??= result.firstInvalidPage;
    }
    return this;
  }
}
