import 'dart:core';

import 'package:clean_android_design/clean_android_design.dart';

class Validator<T> implements IValidation<T> {
  Validator();

  final List<IValidation<T>> _children = <IValidation<T>>[];

  Validator<T> addChild(IValidation<T> validation) {
    _children.add(validation);
    return this;
  }

  Validator<T> addChildren(Iterable<IValidation<T>> validations) {
    _children.addAll(validations);
    return this;
  }

  FieldValidation<T>? getValidation(String validationKey) {
    for (var v in _children) {
      if (v is FieldValidation<T> && v.validationKey == validationKey) return v;
      if (v is Validator<T>) {
        var result = v.getValidation(validationKey);
        if (result != null) return result;
      }
    }
    return null;
  }

  ValidationResult isValid(T item) {
    var result = ValidationResult();

    _children.forEach((validation) => result.merge(validation.isValid(item)));
    return result;
  }

  ReadOnlyValidator<T> readOnly() => ReadOnlyValidator(this);
}

class ReadOnlyValidator<T> {
  final Validator<T> _validator;

  ReadOnlyValidator(this._validator);

  FieldValidation<T>? getValidation(String validationKey) {
    return _validator.getValidation(validationKey);
  }
}
